unit UVCLTuning;

interface

uses Classes, TypInfo;

{        
    .      
    -   -,  ,  ,   .
        ,  
      .      
  This component can do language-localisation for properties of components in the
  project. You can use it for dynamically language-localised properies (not only string-type),
  for example Font, Color and Events. As parameters you can use as "string"-parameters
  and also properties and methods of any existing components of project.


 OwnerClassName/OwnerComponentName.[ComponentName.ComponentName].Property=ComponentName...Property}
type
  TVCLTuning = class (TComponent)
  private
    FTunList: TStringList;
    function GetTL: TStrings;
    procedure SetTL(const Value: TStrings);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure OwnerTuning(AOwner1, AOwner2: TComponent);
    function IsClass(const AClass: TClass; const AClassName: string): boolean;
  published
    property TunList: TStrings read GetTL write SetTL;
  end;

implementation

uses SysUtils;

{ TVCLTuning }

constructor TVCLTuning.Create(AOwner: TComponent);
begin
  inherited;
  FTunList := TStringList.Create;
end;

destructor TVCLTuning.Destroy;
begin
  FTunList.Free;
  inherited;
end;



function TVCLTuning.GetTL: TStrings;
begin
  Result := FTunList;
end;

function TVCLTuning.IsClass(const AClass: TClass; const AClassName: string): boolean;
var P : TClass;
begin
  P := AClass;
  Result := false;
  while (P <> nil)and(P.ClassName <> AClassName) do
     P := P.ClassParent;

  if P <> nil then
    Result := P.ClassName = AClassName;
end;

type
  TFoundKind = (fkNotFound, fkObject, fkProperty, fkMethod);
  TFindRec = record
    IsRec1: boolean;
    OwnerObj : TObject;
    FindName: PChar;
    FoundKind: TFoundKind;
    Obj: TObject;
    PropInfo: PPropInfo;
    Func: pointer;
    UnitName: string[32];
  end;

procedure TVCLTuning.OwnerTuning(AOwner1, AOwner2: TComponent);

  procedure ParseRec(var Rec: TFindRec);
  var i,index : integer; s: string;
  begin
    Rec.Obj := nil;
    Rec.PropInfo := nil;
    Rec.FoundKind := fkNotFound;
    //FillChar(Rec.Method, SizeOf(TMethod), 0);
    s := string(Rec.FindName);
    s := AnsiUpperCase(trim(s));
    //1    (  ,   
    i := pos(']', s);
    if i > 0 then
    begin
      delete(s, i, 1);
      i := pos('[', s);
      try
        index := StrToInt(copy(s, i +1, length(s)));
      except
        index := -1;
      end;
      delete(s, i, length(s));
    end;
    Rec.PropInfo := GetPropInfo(Rec.OwnerObj, s);
    if Rec.PropInfo <> nil then
    begin
      Rec.Obj := Rec.OwnerObj;
      Rec.FoundKind := fkProperty;
      Exit;
    end;
    //2   
    if IsClass(Rec.OwnerObj.ClassType, 'TComponent') then
    begin

      Rec.Obj := TComponent(Rec.OwnerObj).FindComponent(Rec.FindName);
      if Rec.Obj <> nil then
      begin
        //if not Rec.IsRec1 then
        //if ((Rec.UnitName <> '')and(GetTypeData(Rec.Obj.ClassType.ClassInfo).UnitName <> Rec.UnitName)) then
        //begin
        //  Rec.Obj := nil;
        //  Exit;
        //end;
        Rec.FoundKind := fkObject;
        Exit;
      end;
      if Rec.IsRec1 then
      //if ((Rec.UnitName <> '')and(GetTypeData(Rec.Obj.ClassType.ClassInfo).UnitName <> Rec.UnitName)) then
      if (TComponent(Rec.OwnerObj).Name = Rec.FindName)
         or
         IsClass(Rec.OwnerObj.ClassType, Rec.FindName) then
      begin
        Rec.Obj := Rec.OwnerObj;
        Rec.FoundKind := fkObject;
        Exit;
      end;
    end;
    //3   
    Rec.Func := Rec.OwnerObj.MethodAddress(Rec.FindName);
    if Rec.Func <> nil then
    begin
      Rec.Obj := Rec.OwnerObj;
      Rec.FoundKind := fkMethod;
    end;
  end; {end of ParseRec}

  procedure ParseList(const AList: TStrings; var Rec: TFindRec);
  var i,j : integer; S, UnitName : string;
  begin
    //  1.   .  
    //1.       /   /
    //2.   
    //3.       2
    for i := 0 to AList.Count - 1 do
    begin
      S := AList[i];
      Rec.UnitName := '';
      j := pos(':', S);
      if J > 0 then
      begin
        Rec.UnitName := copy(S, 1, J -1);
        delete(S, 1, J);
      end;
      
      Rec.FindName := PChar(S);
      Rec.OwnerObj := Rec.Obj;
      ParseRec(Rec);
      SetLength(S, 0);
      if not Rec.IsRec1 then
      if Rec.FoundKind = fkProperty then
      if Rec.PropInfo^.PropType^^.Kind = tkClass then
        Rec.Obj := GetObjectProp(Rec.Obj, Rec.PropInfo);
      if Rec.FoundKind = fkNotFound then
        Exit;
    end;
  end;

var
  RulNo, i : integer;
  Rule: TStringList;
  S : string;
  M : TMethod;
  Rec1, Rec2: TFindRec;
  SkipRule: boolean;
  C1, C2: TClass;
begin
  Rule := TStringList.Create;
  try
  for RulNo := 0 to FTunList.Count - 1 do
  begin
    S := FTunList[RulNo];
    S := Copy(S, 1, pos('=', S) -1);
    while pos('.', S) > 0 do
      S[pos('.', S)] := #10;
    //1.    
    //2.   
    //3.        
    Rule.Text := S;
    Rec1.Obj := AOwner1;
    Rec1.IsRec1 := True;
    ParseList(Rule, Rec1);
    if Rec1.FoundKind = fkNotFound then Continue;

    //*********************
    //4.   
    //5.  
    //6.     
    S := FTunList[RulNo];
    S := Copy(S, pos('=', S)+1, length(S));
    Rec2.FoundKind := fkNotFound;
    Rec2.IsRec1 := False;
    if S[1] <> '"' then
    begin
      while pos('.', S) > 0 do
        S[pos('.', S)] := #10;
      Rule.Text := S;
      Rec2.Obj := AOwner2;
      ParseList(Rule, Rec2);
    end
    else
    begin
      delete(S, 1, 1);
      if S[Length(S)] = '"' then
        delete(S, Length(S), 1);
    end;
    //     const if Rec2.FoundKind = fkNotFound then Continue;

    //
    //  
    //7.     
    //7.     
    //71.  
    //72.          
    //7.    
    //71.            
    //72.               
    //7.  
    //71.         
    //72.       -    
    case Rec1.FoundKind of  {case1}
      fkNotFound,fkObject: Continue ;
      fkProperty:
        case Rec2.FoundKind of {case2}
          fkNotFound:
            begin
              if Rec1.PropInfo^.PropType^^.Kind in tkProperties -[tkClass] then
              begin
                SetPropValue(Rec1.Obj, Rec1.PropInfo, S);
                Continue;
              end;
            end;
          fkObject:
            if Rec1.PropInfo^.PropType^^.Kind in [tkClass] then
              SetObjectProp(Rec1.Obj, Rec1.PropInfo, Rec2.Obj);
          fkProperty:
            begin
            if (Rec1.PropInfo^.PropType^^.Kind in [tkMethod])
            and (Rec2.PropInfo^.PropType^^.Kind in [tkMethod]) then
            begin
              SetMethodProp(Rec1.Obj, Rec1.PropInfo, GetMethodProp(Rec2.Obj, Rec2.PropInfo));
              Continue;
            end;

            if (Rec1.PropInfo^.PropType^^.Kind in [tkClass])
            and (Rec2.PropInfo^.PropType^^.Kind in [tkClass]) then
            begin
              C1 := GetTypeData(Rec1.PropInfo^.PropType^).ClassType;
              C2 := GetTypeData(Rec2.PropInfo^.PropType^).ClassType;
              while C2 <> nil do
              begin
                if IsClass(C2, C1.ClassName) then
                begin
                  C2 := nil;
                  C1 := nil;
                  SetObjectProp(Rec1.Obj, Rec1.PropInfo, GetObjectProp(Rec2.Obj, Rec2.PropInfo));
                end
                else
                  C2 := C2.ClassParent;
              end;
              Continue;
            end;
            end;
          fkMethod:
          begin
            M.Code := Rec2.Func;    
            M.Data := Rec2.Obj;
            SetMethodProp(Rec1.Obj, Rec1.PropInfo, M);
          end;
        end; {case2}

//      fkMethod:
//        begin
//          M.Code := Rec2.Func;
//          M.Data := Rec2.Obj;
//          SetMethodProp(Rec1.Obj, Rec1.PropInfo, M);
//        end;
    end; {case1}

    //***********************

  end; {for}
  finally
    Rule.Free;
  end;

end;

procedure TVCLTuning.SetTL(const Value: TStrings);
begin
  FTunList.Assign(Value);
end;

end.
